/**
	Definiciones para un programa
	para deformar un simple sistema resorte-masa-amortiguador en 2D
	(este programa usa primivitas de OpenGL y  Qt v. 4)

	Fraga  8/12/2010
	Fraga 28/01/2012
**/
#ifndef SPRING_H
#define SPRING_H

#include <QWidget>
#include <QtOpenGL>
#include <QTimer>
#include <QDoubleSpinBox>
#include <math.h>

class Spring: public QGLWidget
{
	Q_OBJECT  // Debe incluirse para usar seales/ranuras de Qt
public:
	Spring( QWidget *parent=0 );

	void muestraResortes( void );
	void Inicializa( void );
	QSizePolicy sizePolicy() const;

public slots:
	void resetea( void ) {
		timer->stop( );
		Inicializa( );

		updateGL();
	}
	void escogeResorte( int m ){
		n = m-1;
		updateGL();
	}
	void poneFuerza( double fuerza ) { 
		r.f = 11.11 * fuerza;
		// fprintf( stderr, "f = %lf\n", r.f );
	}
	void deforma( void ) { 
		r.rs[n].fx = r.f * cos ( r.theta );
		r.rs[n].fy = r.f * sin ( r.theta );

		timer->start( 33 );
	}
	void quitaFuerza( void ) { 
		r.f = 0.0;
		r.rs[n].fx = r.rs[n].fy = 0.0;
	}
	void poneAngulo( double a ) { 
		r.theta = a * M_PI/180.0;
	}
	void unPasoDeformacion( void );

protected:
	void initializeGL( void );
	void resizeGL( int w, int h );
	void paintGL( void );
 
 
private:
	QTimer *timer;

	int windowWidth, windowHeight;   // canvas width and height

	float yrot, xrot;  // Encene rotation

	double angle;

	// Una estructura para almacenar el estado de un nodo.
	// Este es un sistema MRA 
	typedef struct mra {
		double fx, fy;
		double x_1, x, x1;
		double y_1, y, y1;
		double px, py;
	} MRA;

	// Resorte en dos dimensiones
	typedef struct mra2d {
		double k, m, b;
		double dt;  // Delta i inicial: dt = 0.0333333333;
		double k1, k2, k3, k4;
		double f, theta;

		MRA rs[3];
	} MRA2D;

	MRA2D r; 
	double p0x, p0y, pnx, pny;
	int n;
};


#endif // SPRING_H
